﻿param ([string[]] $rootPath)
## general variables and prep
#$rootPath = $PSScriptRoot # apparently Powershell 3.0 isn't on all Win7 versions...
$version = '1.04'
$rootPath = $rootPath -replace '"',''
[void][Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic')
## core functions
#### ---------------------------------------
#### clean all trash files
#### ---------------------------------------
function Log{
param($level, $blob)
	if($level -eq 'W'){
		write-host "[WARNING]`t" -NoNewLine -ForegroundColor Yellow
	}
	if($level -eq 'E'){
		write-host "[ERROR]`t" -NoNewLine -ForegroundColor Red
	}
	if($level -eq 'OK'){
		write-host "[OK]`t" -NoNewLine -ForegroundColor Green
	}
	write-host $blob
}

function CleanDir{
    del "$rootPath\Temp\*.*" -Force
}
#### ---------------------------------------
#### searching for character block in a file
#### ---------------------------------------
function searchPattern{
    param($file)    
    $hexPattern =  [Regex] '\x6E\x00{10}\x3F'
    $Encoding = [Text.Encoding]::GetEncoding(28591)
    $fileStream = New-Object IO.FileStream -ArgumentList (Resolve-Path $file), 'Open', 'Read'
    $StreamReader = New-Object IO.StreamReader -ArgumentList $fileStream, $Encoding
    $BinaryText = $StreamReader.ReadToEnd()
    $m = $hexPattern.Matches($BinaryText)
    $StreamReader.Close()
    $fileStream.Close()
    return $m
}
#### ---------------------------------------
#### dumping the whole character block into a temp file
#### ---------------------------------------
function grabCharBlock{
    param($file,$index)    
    $Encoding = [Text.Encoding]::GetEncoding(28591)
    $fileStream = New-Object IO.FileStream -ArgumentList (Resolve-Path $file), 'Open', 'Read'    
    $BinaryReader = New-Object IO.BinaryReader -ArgumentList $fileStream, $Encoding
    $BinaryReader.BaseStream.Position = $index
    $tmp = $BinaryReader.ReadBytes(8804)    
    $BinaryReader.Close()
    $fileStream.Close()
    $dump = $((gci $file).Name)
    [System.IO.File]::WriteAllBytes("$rootPath\Temp\$dump.dump", $tmp)
    return "$rootPath\Temp\$dump.dump"    
}
#### ---------------------------------------
#### overwriting existing character block with a new one
#### USE ON A COPY, NEVER ON ORIGINAL FILE
#### ---------------------------------------
function writeCharBlock{
    param($file,$index, $charBlock)
    $Encoding = [Text.Encoding]::GetEncoding(28591)    
    
    #one with charblock
    $dumpStream = New-Object IO.FileStream -ArgumentList (Resolve-Path $charblock), 'Open', 'Read'
    $dumpReader = New-Object IO.BinaryReader -ArgumentList $dumpStream, $Encoding
    $charData = $dumpReader.ReadBytes(8804)
    $dumpReader.Close()
    $dumpStream.Close()
    
    #one we will write to
    $fileStream = New-Object IO.FileStream -ArgumentList (Resolve-Path $file), 'Open', 'ReadWrite'    
    $BinaryWriter = New-Object IO.BinaryWriter -ArgumentList $fileStream, $Encoding
    
    #setting position to index of charblock in a writable file
    $BinaryWriter.BaseStream.Position = $index
    $BinaryWriter.Write($charData)
    $fileStream.Close()
    $BinaryWriter.Close()
}
#### ---------------------------------------
#### checking if file actually changed
#### ---------------------------------------
function validateFile{
    param($workfile,$orgfile)
    if((Get-FileHash $workfile).hash  -ne (Get-FileHash $orgfile).hash){
        return $true
    } else {
        return $false
    }
}
#### ---------------------------------------
#### starting actual work
#### ---------------------------------------
function processFile{
    param($file,$newfile)
    if($newfile -eq ''){
		$newfile = "$rootPath\srgsave_00.srgs_pcDEFAULT"
		COPY "$rootPath\AUTOSAVE_ONLY_savedir.srgd_pc" "$rootPath\Out\savedir.srgd_pc"
	}
    
    $blocks = searchPattern $file
    if($blocks.Count -eq 1){
        Log OK "One pattern found"
        $i = 0        
    } else {
        if($blocks.Count -eq 2){
            Log W "Two patterns found"
            $i = 1            
        } else {
            Log E "Wrong or unsupported file"       
            write-host "Terminating script"
            CleanDir
            break
        }
    }
    $patternIndex = $blocks[$i].Index    
    $dumpBlock = grabCharBlock $file $patternIndex
    ##copy save to workdir and leave original untouched
    $workfile = "$rootPath\Temp\srgsave_00.srgs_pc"
    copy $newfile $workfile
    $blocksGOH = searchPattern $workfile
    if(!($blocksGOH.Count -eq 2)){
        ## workaround for hard-coded address 7FA4 / 32676
		Log W "Boss entry not found in GOOH file"
		Log W "Attempting to write at hardcoded address [EXPERIMENTAL]"
		writeCharBlock $workfile 32676 $dumpBlock
		if(ValidateFile $workfile $newfile){
			copy $workfile "$rootPath\Out\srgsave_00.srgs_pc"
		} else {
			Log E "Invalid GOOH file! Cannot copy data."
			write-host "Terminating script"
			CleanDir
			break
		}
        
    } else {
        $patternGOH = $blocksGOH[1].Index
        writeCharBlock $workfile $patternGOH $dumpBlock
        if(ValidateFile $workfile $newfile){
            copy $workfile "$rootPath\Out\srgsave_00.srgs_pc"            
        } else {
            Log E "Operation unsuccessful."
            write-host "Terminating script"
			CleanDir
			break
        }
    }    
    Log OK "All Good!"
	write-host "You can find your modified file here:"
	write-host "$rootPath\Out" -ForegroundColor Cyan
	write-host ""
	write-host "!!REMEMBER TO BACKUP YOUR ORIGINAL SAVES BEFORE REPLACING THEM WITH MODIFIED ONES!!" -ForegroundColor Yellow
	CleanDir
}
#### ---------------------------------------
#### base info for user
#### ---------------------------------------
function introFrame{    
    write-host '====================================' -ForegroundColor Magenta
    write-host "BOSS IMPORT script for SR3/4 -> GOOH v.$version"
    write-host 'please report any problems in this thread:'
    write-host 'https://www.saintsrowmods.com/forum/threads/import-boss-from-sr4-to-gooh.14763/' -ForegroundColor Green
    write-host '====================================' -ForegroundColor Magenta
    write-host '!!ALWAYS BACKUP YOUR SAVES AND USE AT YOUR OWN RISK!!' -ForegroundColor Yellow
    write-host '====================================' -ForegroundColor Magenta
    write-host ''    
}
#### ---------------------------------------
#### USER INPUT PART - Paths
#### ---------------------------------------
introFrame
$title = 'Import From Save...'
$msg = 'Path to S3/S4 save to import The Boss FROM:'
$saveFrom = [Microsoft.VisualBasic.Interaction]::InputBox($msg, $title)

$title = '[OPTIONAL] Import To Save...'
$msg = '(Optional) path to the existing GOOH save. Leave empty for a new save to be created.'
$saveTo = [Microsoft.VisualBasic.Interaction]::InputBox($msg, $title)

write-host 'USER PATHS:' -ForegroundColor Cyan
write-host "import The Boss from:`t" -NoNewline
write-host $saveFrom -ForegroundColor Green
write-host "to the save:`t" -NoNewline
if($saveTo -eq ''){    
    write-host "NEW FILE" -ForegroundColor Yellow
} else {
    write-host $saveTo -ForegroundColor Green
}
write-host '' 
processFile $saveFrom $saveTo